@@ -1,4 +1,8 @@
-{-# Language OverloadedStrings, RecordWildCards, StandaloneDeriving #-}
+{-# LANGUAGE CPP #-}
+{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE RecordWildCards #-}
+{-# LANGUAGE StandaloneDeriving #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
 module Instances where
@@ -6,55 +10,36 @@ module Instances where
 import Types
 import Data.Function (on)
 import Control.Monad
-import Test.QuickCheck (Arbitrary(..), Gen, choose, oneof, elements)
-import Data.Time.Clock (DiffTime, UTCTime(..), picosecondsToDiffTime)
-import Data.Fixed (Pico)
-import Data.Time (ZonedTime(..), LocalTime(..), TimeZone(..),
-                  hoursToTimeZone, Day(..), TimeOfDay(..),
-                  NominalDiffTime)
-import qualified Data.Text as T
-import qualified Data.Map as Map
-import Data.Text (Text)
+import Test.QuickCheck (Arbitrary(..), getNonNegative, elements,
+                        listOf1, oneof, resize)
+import Data.Time.Clock (UTCTime(..))
+import Data.Time (ZonedTime(..), TimeZone(..))
+import Data.List.NonEmpty (NonEmpty(..))
+import Data.Version
 import Data.Aeson.Types
+import Control.Applicative (Const(..))
+import Data.Tagged (Tagged(..))
+import Data.Proxy (Proxy(..))
 import Control.Applicative
 import Functions
 
+import Data.Orphans ()
+import Test.QuickCheck.Instances ()
+
+#if !MIN_VERSION_base(4,8,0) && !MIN_VERSION_QuickCheck(2,8,3)
+import Numeric.Natural
+#endif
+
 -- "System" types.
 
-instance Arbitrary Text where
-    arbitrary = T.pack <$> arbitrary
-
-instance (Ord k, Arbitrary k, Arbitrary v) => Arbitrary (Map.Map k v) where
-    arbitrary = Map.fromList <$> arbitrary
-
-instance Arbitrary LocalTime where
-    arbitrary = return $ LocalTime (ModifiedJulianDay 1) (TimeOfDay 1 2 3)
-
-instance Arbitrary TimeZone where
-    arbitrary = do
-      offset <- choose (0,2) :: Gen Int
-      return $ hoursToTimeZone offset
-
-instance Arbitrary Day where
-    arbitrary = ModifiedJulianDay `liftM` arbitrary
-
-instance Arbitrary DiffTime where
-    arbitrary = (picosecondsToDiffTime . (* 1000000000)) <$>
-                choose (0, 86400000)
-
-instance Arbitrary UTCTime where
-    arbitrary = liftM2 UTCTime arbitrary arbitrary
-
 instance Arbitrary DotNetTime where
     arbitrary = DotNetTime `liftM` arbitrary
+    shrink = map DotNetTime . shrink . fromDotNetTime
 
-instance Arbitrary ZonedTime where
-    arbitrary = liftM2 ZonedTime arbitrary arbitrary
-
-instance Arbitrary NominalDiffTime where
-    arbitrary = realToFrac <$> (arbitrary :: Gen Pico)
-
-deriving instance Eq ZonedTime
+-- | Compare timezone part only on 'timeZoneMinutes'
+instance Eq ZonedTime where
+  ZonedTime a (TimeZone a' _ _) == ZonedTime b (TimeZone b' _ _) =
+    a == b && a' == b'
 
 -- Compare equality to within a millisecond, allowing for rounding
 -- error (ECMA 262 requires milliseconds to rounded to zero, not
@@ -148,8 +133,41 @@ instance Arbitrary a => Arbitrary (SomeType a) where
                       , Record  <$> arbitrary <*> arbitrary <*> arbitrary
                       ]
 
+instance Arbitrary (GADT String) where
+    arbitrary = GADT <$> arbitrary
+
 instance ApproxEq Char where
     (=~) = (==)
 
 instance (ApproxEq a) => ApproxEq [a] where
     a =~ b = length a == length b && all (uncurry (=~)) (zip a b)
+
+instance Arbitrary Version where
+    arbitrary = makeVersion . fmap getNonNegative <$> resize 4 (listOf1 arbitrary)
+
+instance Arbitrary a => Arbitrary (NonEmpty a) where
+    arbitrary = (:|) <$> arbitrary <*> arbitrary
+
+-- Version tags are deprecated, so we avoid using them in the Arbitrary
+-- instance. However, the recommended constructor 'makeVersion' is not
+-- exported by "Data.Version" until base-4.8.0.0. For previous versions,
+-- a definition is given below.
+
+#if !MIN_VERSION_base(4,8,0)
+makeVersion :: [Int] -> Version
+makeVersion b = Version b []
+#endif
+
+#if !MIN_VERSION_base(4,8,0) && !MIN_VERSION_QuickCheck(2,8,3)
+instance Arbitrary Natural where
+  arbitrary = fromInteger . abs <$> arbitrary
+#endif
+
+instance Arbitrary (Proxy a) where
+    arbitrary = pure Proxy
+
+instance Arbitrary b => Arbitrary (Tagged a b) where
+    arbitrary = Tagged <$> arbitrary
+
+instance Arbitrary a => Arbitrary (Const a b) where
+    arbitrary = Const <$> arbitrary
